<?php
namespace Teknisa\CUS\Modules\Relatorio\Controllers;

use Zeedhi\Framework\Report\ReportService;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Zeedhi\Framework\DTO\Response\Error;

class Cus03000AnaliseGerencialCardCong extends \Zeedhi\Framework\Controller\Report {

	protected $reportService;
	protected $crudService;
	protected $dataSourceManager;
	protected $novoCodigo;
	protected $entityManager;
	protected $birtStrategy;
	protected $connection;
	protected $environment;

	public function __construct($reportService, $crudService, $dataSourceManager, $novoCodigo, $entityManager, $birtStrategy, $util, $connection, $environment) {
		$this->reportService    = $reportService;
		$this->crudService      = $crudService;
		$this->dataSourceManager= $dataSourceManager;
		$this->novoCodigo       = $novoCodigo;
		$this->entityManager    = $entityManager;
		$this->birtStrategy     = $birtStrategy;
		$this->util             = $util;
		$this->connection		= $connection;
		$this->environment		= $environment;
	}

	protected $reportMapping = array (
		'CUS03000' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
				'NRORG' 		 => 'NRORG',
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT'
			)
		),
		'CUS03001' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
				'NRORG' 		 => 'NRORG',
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT'
			)
		),
		'CUS03002' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
				'NRORG' 		 => 'NRORG',
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT'
			)
		),
		'CUS03003' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03004' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03005' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03006' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03007' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03008' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03009' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03010' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03011' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		),
		'CUS03012' => array (
			'strategy' => 'BIRT',
			'parameterMapping' 		=> array (
			    'CDTIPOCUSTOREAL'=> 'CDTIPOCUSTOREAL',
			    'CDSERVFILI'     => 'CDSERVFILI',
			    'CDFILIAL'       => 'CDFILIAL',
			    'DTINICIAL'      => 'DTINICIAL',
			    'DTFINAL'        => 'DTFINAL',
			    'NIVELINDICE'    => 'NIVELINDICE',
			    'CDARVPRODINI'   => 'CDARVPRODINI',
			    'CDARVPRODFIN'   => 'CDARVPRODFIN',
			    'CDGRUPFILI'     => 'CDGRUPFILI',
			    'IDORDENACAO'    => 'IDORDENACAO',
			    'DSTIPOCUSTO'    => 'DSTIPOCUSTO',
			    'CDTIPOPRAT'	 => 'CDTIPOPRAT',
			    'NRORG' 		 => 'NRORG'
			)
		)
	);

	public function beforeProcessReport(DTO\Request\Row $request, DTO\Response $response) {
		$row = $request->getRow();
		$row['NRORG']      = $this->environment->getNrorgTrab();
		$row['CDTIPOPRAT'] = $this->criarListaValores($row['CDTIPOPRAT']);
		$row['CDFILIAL']   = $this->criarListaValores($row['CDFILIAL']);
		$row['CDSERVFILI'] = $this->criarListaValores($row['CDSERVFILI']);
	}
	
	public function afterProcessReport(DTO\Request\Row $request, DTO\Response $response, $report) {
  		$row = $request->getRow();
		$this->apagarListaValores($row['CDTIPOPRAT']);
		$this->apagarListaValores($row['CDFILIAL']);
		$this->apagarListaValores($row['CDSERVFILI']);
	}

    //insere lista no banco
    public function insertLista($nrlista, $cdlista) {
        $params = array(
            "NRLISTA" => $nrlista,
            "CDLISTA" => $cdlista
        );
        
        $sql = "INSERT INTO LISTA (NRLISTA, CDLISTA) VALUES (:NRLISTA, :CDLISTA)";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //apaga lista do banco
    public function deleteLista($nrlista) {
        $params = array(
            "NRLISTA" => $nrlista
        );
        
        $sql = "DELETE FROM LISTA WHERE NRLISTA =:NRLISTA";
        $query = $this->connection->prepare($sql);
        $query->execute($params);
    }
    
    //cria lista de valores recebendo string como parametro
    public function criarListaValores($valores){
        if(strlen($valores) <= 4000){
            return $valores;
        }else{
            $this->entityManager->beginTransaction();
            $lista = explode('_',$valores);
            $nrlista = $this->novoCodigo->getCodeByTableName('LISTA', $this->environment->getNrorgTrab(), 1, 10, false);
            try{
                foreach($lista as $key){
                    $this->insertLista($nrlista, $key);
                }
                $this->entityManager->commit();
                return "_NRLISTA=".$nrlista;
            } catch(\Exception $e){
                $this->entityManager->rollback();
                throw $e;
            }
        }
    }
    
    //apaga lista de valores recebendo string como parametro
    public function apagarListaValores($valores){
        try{
            if(substr($valores, 0,9) == "_NRLISTA="){
                $nrlista = substr($valores, 9);
                $this->deleteLista($nrlista);
            }
        } catch(\Exception $e){
            throw $e;
        }
    }
    
	private function insereLista($nrlista, $filiserv) {
		$rows = array();
		foreach($filiserv as $filialservico){
			array_push($rows,
				array(
					"NRLISTA" 	=> $nrlista,
					"CDLISTA" 	=> $filialservico,
					"__is_new"  => true
				)
			);
		}
		$dataSet = new DataSet(null, $rows);
		$request = new DTO\Request\DataSet($dataSet, '', '', '');
		$rowsToSave = $this->crudService->getRowsToSave($request, 'lista');
		$this->crudService->save('lista', $rowsToSave);
	}

}